@@ -38,6 +38,7 @@ struct xdg_positioner_constraint_adjustment;
 class xdg_surface_t;
 enum class xdg_surface_error : uint32_t;
 class xdg_toplevel_t;
+enum class xdg_toplevel_error : uint32_t;
 struct xdg_toplevel_resize_edge;
 enum class xdg_toplevel_state : uint32_t;
 class xdg_popup_t;
@@ -80,7 +81,7 @@ private:
     std::function<void(uint32_t)> clock_id;
   };
 
-  static int dispatcher(uint32_t opcode, std::vector<detail::any> args, std::shared_ptr<detail::events_base_t> e);
+  static int dispatcher(uint32_t opcode, const std::vector<detail::any>& args, const std::shared_ptr<detail::events_base_t>& e);
 
   presentation_t(proxy_t const &wrapped_proxy, construct_proxy_wrapper_tag /*unused*/);
 
@@ -109,7 +110,7 @@ public:
         presentation_feedback interface.
       
   */
-  presentation_feedback_t feedback(surface_t surface);
+  presentation_feedback_t feedback(surface_t const& surface);
 
   /** \brief Minimum protocol version required for the \ref feedback function
   */
@@ -193,7 +194,7 @@ private:
     std::function<void()> discarded;
   };
 
-  static int dispatcher(uint32_t opcode, std::vector<detail::any> args, std::shared_ptr<detail::events_base_t> e);
+  static int dispatcher(uint32_t opcode, const std::vector<detail::any>& args, const std::shared_ptr<detail::events_base_t>& e);
 
   presentation_feedback_t(proxy_t const &wrapped_proxy, construct_proxy_wrapper_tag /*unused*/);
 
@@ -291,35 +292,7 @@ public:
         These flags provide information about how the presentation of
         the related content update was done. The intent is to help
         clients assess the reliability of the feedback and the visual
-        quality with respect to possible tearing and timings. The
-        flags are:
-
-        VSYNC:
-        The presentation was synchronized to the "vertical retrace" by
-        the display hardware such that tearing does not happen.
-        Relying on user space scheduling is not acceptable for this
-        flag. If presentation is done by a copy to the active
-        frontbuffer, then it must guarantee that tearing cannot
-        happen.
-
-        HW_CLOCK:
-        The display hardware provided measurements that the hardware
-        driver converted into a presentation timestamp. Sampling a
-        clock in user space is not acceptable for this flag.
-
-        HW_COMPLETION:
-        The display hardware signalled that it started using the new
-        image content. The opposite of this is e.g. a timer being used
-        to guess when the display hardware has switched to the new
-        image content.
-
-        ZERO_COPY:
-        The presentation of this update was done zero-copy. This means
-        the buffer from the client was given to display hardware as
-        is, without copying it. Compositing with OpenGL counts as
-        copying, even if textured directly from the client buffer.
-        Possible zero-copy cases include direct scanout of a
-        fullscreen surface and a surface on a hardware overlay.
+        quality with respect to possible tearing and timings.
       
   */
 struct presentation_feedback_kind : public detail::bitfield<4, 1>
@@ -328,13 +301,9 @@ struct presentation_feedback_kind : public detail::bitfield<4, 1>
     : detail::bitfield<4, 1>(b) {}
   presentation_feedback_kind(const uint32_t value)
     : detail::bitfield<4, 1>(value) {}
-  /** \brief presentation was vsync'd */
   static const detail::bitfield<4, 1> vsync;
-  /** \brief hardware provided the presentation timestamp */
   static const detail::bitfield<4, 1> hw_clock;
-  /** \brief hardware signalled the start of the presentation */
   static const detail::bitfield<4, 1> hw_completion;
-  /** \brief presentation was done zero-copy */
   static const detail::bitfield<4, 1> zero_copy;
 };
 
@@ -356,7 +325,7 @@ private:
   {
   };
 
-  static int dispatcher(uint32_t opcode, std::vector<detail::any> args, std::shared_ptr<detail::events_base_t> e);
+  static int dispatcher(uint32_t opcode, const std::vector<detail::any>& args, const std::shared_ptr<detail::events_base_t>& e);
 
   viewporter_t(proxy_t const &wrapped_proxy, construct_proxy_wrapper_tag /*unused*/);
 
@@ -381,7 +350,7 @@ public:
 	protocol error is raised.
       
   */
-  viewport_t get_viewport(surface_t surface);
+  viewport_t get_viewport(surface_t const& surface);
 
   /** \brief Minimum protocol version required for the \ref get_viewport function
   */
@@ -471,7 +440,7 @@ private:
   {
   };
 
-  static int dispatcher(uint32_t opcode, std::vector<detail::any> args, std::shared_ptr<detail::events_base_t> e);
+  static int dispatcher(uint32_t opcode, const std::vector<detail::any>& args, const std::shared_ptr<detail::events_base_t>& e);
 
   viewport_t(proxy_t const &wrapped_proxy, construct_proxy_wrapper_tag /*unused*/);
 
@@ -569,7 +538,7 @@ private:
     std::function<void(uint32_t)> ping;
   };
 
-  static int dispatcher(uint32_t opcode, std::vector<detail::any> args, std::shared_ptr<detail::events_base_t> e);
+  static int dispatcher(uint32_t opcode, const std::vector<detail::any>& args, const std::shared_ptr<detail::events_base_t>& e);
 
   xdg_wm_base_t(proxy_t const &wrapped_proxy, construct_proxy_wrapper_tag /*unused*/);
 
@@ -602,7 +571,9 @@ public:
 
 	This creates an xdg_surface for the given surface. While xdg_surface
 	itself is not a role, the corresponding surface may only be assigned
-	a role extending xdg_surface, such as xdg_toplevel or xdg_popup.
+	a role extending xdg_surface, such as xdg_toplevel or xdg_popup. It is
+	illegal to create an xdg_surface for a wl_surface which already has an
+	assigned role and this will result in a protocol error.
 
 	This creates an xdg_surface for the given surface. An xdg_surface is
 	used as basis to define a role to a given surface, such as xdg_toplevel
@@ -613,7 +584,7 @@ public:
 	xdg_surface is and how it is used.
       
   */
-  xdg_surface_t get_xdg_surface(surface_t surface);
+  xdg_surface_t get_xdg_surface(surface_t const& surface);
 
   /** \brief Minimum protocol version required for the \ref get_xdg_surface function
   */
@@ -702,7 +673,7 @@ private:
   {
   };
 
-  static int dispatcher(uint32_t opcode, std::vector<detail::any> args, std::shared_ptr<detail::events_base_t> e);
+  static int dispatcher(uint32_t opcode, const std::vector<detail::any>& args, const std::shared_ptr<detail::events_base_t>& e);
 
   xdg_positioner_t(proxy_t const &wrapped_proxy, construct_proxy_wrapper_tag /*unused*/);
 
@@ -769,7 +740,7 @@ public:
 	edge, or in the center of the anchor rectangle if no edge is specified.
       
   */
-  void set_anchor(xdg_positioner_anchor anchor);
+  void set_anchor(xdg_positioner_anchor const& anchor);
 
   /** \brief Minimum protocol version required for the \ref set_anchor function
   */
@@ -786,7 +757,7 @@ public:
 	gravity specified.
       
   */
-  void set_gravity(xdg_positioner_gravity gravity);
+  void set_gravity(xdg_positioner_gravity const& gravity);
 
   /** \brief Minimum protocol version required for the \ref set_gravity function
   */
@@ -810,7 +781,7 @@ public:
 	The default adjustment is none.
       
   */
-  void set_constraint_adjustment(xdg_positioner_constraint_adjustment constraint_adjustment);
+  void set_constraint_adjustment(xdg_positioner_constraint_adjustment const& constraint_adjustment);
 
   /** \brief Minimum protocol version required for the \ref set_constraint_adjustment function
   */
@@ -839,6 +810,71 @@ public:
   */
   static constexpr std::uint32_t set_offset_since_version = 1;
 
+  /** \brief continuously reconstrain the surface
+
+	When set reactive, the surface is reconstrained if the conditions used
+	for constraining changed, e.g. the parent window moved.
+
+	If the conditions changed and the popup was reconstrained, an
+	xdg_popup.configure event is sent with updated geometry, followed by an
+	xdg_surface.configure event.
+      
+  */
+  void set_reactive();
+
+  /** \brief Minimum protocol version required for the \ref set_reactive function
+  */
+  static constexpr std::uint32_t set_reactive_since_version = 3;
+
+  /** \brief Check whether the \ref set_reactive function is available with
+      the currently bound version of the protocol
+  */
+  bool can_set_reactive() const;
+
+  /** \brief 
+      \param parent_width future window geometry width of parent
+      \param parent_height future window geometry height of parent
+
+	Set the parent window geometry the compositor should use when
+	positioning the popup. The compositor may use this information to
+	determine the future state the popup should be constrained using. If
+	this doesn't match the dimension of the parent the popup is eventually
+	positioned against, the behavior is undefined.
+
+	The arguments are given in the surface-local coordinate space.
+      
+  */
+  void set_parent_size(int32_t parent_width, int32_t parent_height);
+
+  /** \brief Minimum protocol version required for the \ref set_parent_size function
+  */
+  static constexpr std::uint32_t set_parent_size_since_version = 3;
+
+  /** \brief Check whether the \ref set_parent_size function is available with
+      the currently bound version of the protocol
+  */
+  bool can_set_parent_size() const;
+
+  /** \brief set parent configure this is a response to
+      \param serial serial of parent configure event
+
+	Set the serial of an xdg_surface.configure event this positioner will be
+	used in response to. The compositor may use this information together
+	with set_parent_size to determine what future state the popup should be
+	constrained using.
+      
+  */
+  void set_parent_configure(uint32_t serial);
+
+  /** \brief Minimum protocol version required for the \ref set_parent_configure function
+  */
+  static constexpr std::uint32_t set_parent_configure_since_version = 3;
+
+  /** \brief Check whether the \ref set_parent_configure function is available with
+      the currently bound version of the protocol
+  */
+  bool can_set_parent_configure() const;
+
 };
 
 /** \brief 
@@ -940,6 +976,11 @@ struct xdg_positioner_constraint_adjustment : public detail::bitfield<6, 8>
       manipulate a buffer prior to the first xdg_surface.configure call must
       also be treated as errors.
 
+      After creating a role-specific object and setting it up, the client must
+      perform an initial commit without any buffer attached. The compositor
+      will reply with an xdg_surface.configure event. The client must
+      acknowledge it and is then allowed to attach a buffer to map the surface.
+
       Mapping an xdg_surface-based role surface is defined as making it
       possible for the surface to be shown by the compositor. Note that
       a mapped surface is not guaranteed to be visible once it is mapped.
@@ -953,7 +994,8 @@ struct xdg_positioner_constraint_adjustment : public detail::bitfield<6, 8>
 
       A newly-unmapped surface is considered to have met condition (1) out
       of the 3 required conditions for mapping a surface if its role surface
-      has not been destroyed.
+      has not been destroyed, i.e. the client must perform the initial commit
+      again before attaching a buffer.
     
 */
 class xdg_surface_t : public proxy_t
@@ -964,7 +1006,7 @@ private:
     std::function<void(uint32_t)> configure;
   };
 
-  static int dispatcher(uint32_t opcode, std::vector<detail::any> args, std::shared_ptr<detail::events_base_t> e);
+  static int dispatcher(uint32_t opcode, const std::vector<detail::any>& args, const std::shared_ptr<detail::events_base_t>& e);
 
   xdg_surface_t(proxy_t const &wrapped_proxy, construct_proxy_wrapper_tag /*unused*/);
 
@@ -1008,7 +1050,7 @@ public:
 	xdg_popup is and how it is used.
       
   */
-  xdg_popup_t get_popup(xdg_surface_t parent, xdg_positioner_t positioner);
+  xdg_popup_t get_popup(xdg_surface_t const& parent, xdg_positioner_t const& positioner);
 
   /** \brief Minimum protocol version required for the \ref get_popup function
   */
@@ -1135,7 +1177,11 @@ enum class xdg_surface_error : uint32_t
       by the compositor until it is explicitly mapped again.
       All active operations (e.g., move, resize) are canceled and all
       attributes (e.g. title, state, stacking, ...) are discarded for
-      an xdg_toplevel surface when it is unmapped.
+      an xdg_toplevel surface when it is unmapped. The xdg_toplevel returns to
+      the state it had right after xdg_surface.get_toplevel. The client
+      can re-map the toplevel by perfoming a commit without any buffer
+      attached, waiting for a configure event and handling it as usual (see
+      xdg_surface description).
 
       Attaching a null buffer to a toplevel unmaps the surface.
     
@@ -1147,9 +1193,10 @@ private:
   {
     std::function<void(int32_t, int32_t, array_t)> configure;
     std::function<void()> close;
+    std::function<void(int32_t, int32_t)> configure_bounds;
   };
 
-  static int dispatcher(uint32_t opcode, std::vector<detail::any> args, std::shared_ptr<detail::events_base_t> e);
+  static int dispatcher(uint32_t opcode, const std::vector<detail::any>& args, const std::shared_ptr<detail::events_base_t>& e);
 
   xdg_toplevel_t(proxy_t const &wrapped_proxy, construct_proxy_wrapper_tag /*unused*/);
 
@@ -1185,7 +1232,7 @@ public:
 	parent surface.
       
   */
-  void set_parent(xdg_toplevel_t parent);
+  void set_parent(xdg_toplevel_t const& parent);
 
   /** \brief Minimum protocol version required for the \ref set_parent function
   */
@@ -1203,7 +1250,7 @@ public:
 	The string must be encoded in UTF-8.
       
   */
-  void set_title(std::string title);
+  void set_title(std::string const& title);
 
   /** \brief Minimum protocol version required for the \ref set_title function
   */
@@ -1237,7 +1284,7 @@ public:
 	[0] http://standards.freedesktop.org/desktop-entry-spec/
       
   */
-  void set_app_id(std::string app_id);
+  void set_app_id(std::string const& app_id);
 
   /** \brief Minimum protocol version required for the \ref set_app_id function
   */
@@ -1262,7 +1309,7 @@ public:
 	like a button press, key press, or touch down event.
       
   */
-  void show_window_menu(seat_t seat, uint32_t serial, int32_t x, int32_t y);
+  void show_window_menu(seat_t const& seat, uint32_t serial, int32_t x, int32_t y);
 
   /** \brief Minimum protocol version required for the \ref show_window_menu function
   */
@@ -1290,7 +1337,7 @@ public:
 	that the device focus will return when the move is completed.
       
   */
-  void move(seat_t seat, uint32_t serial);
+  void move(seat_t const& seat, uint32_t serial);
 
   /** \brief Minimum protocol version required for the \ref move function
   */
@@ -1325,15 +1372,16 @@ public:
 	guarantee that the device focus will return when the resize is
 	completed.
 
-	The edges parameter specifies how the surface should be resized,
-	and is one of the values of the resize_edge enum. The compositor
-	may use this information to update the surface position for
-	example when dragging the top left corner. The compositor may also
-	use this information to adapt its behavior, e.g. choose an
-	appropriate cursor image.
+	The edges parameter specifies how the surface should be resized, and
+	is one of the values of the resize_edge enum. Values not matching
+	a variant of the enum will cause a protocol error. The compositor
+	may use this information to update the surface position for example
+	when dragging the top left corner. The compositor may also use
+	this information to adapt its behavior, e.g. choose an appropriate
+	cursor image.
       
   */
-  void resize(seat_t seat, uint32_t serial, xdg_toplevel_resize_edge edges);
+  void resize(seat_t const& seat, uint32_t serial, xdg_toplevel_resize_edge const& edges);
 
   /** \brief Minimum protocol version required for the \ref resize function
   */
@@ -1519,7 +1567,7 @@ public:
 	visible below the fullscreened surface.
       
   */
-  void set_fullscreen(output_t output);
+  void set_fullscreen(output_t const& output);
 
   /** \brief Minimum protocol version required for the \ref set_fullscreen function
   */
@@ -1612,6 +1660,38 @@ public:
   */
   std::function<void()> &on_close();
 
+  /** \brief recommended window geometry bounds
+      \param width 
+      \param height 
+
+	The configure_bounds event may be sent prior to a xdg_toplevel.configure
+	event to communicate the bounds a window geometry size is recommended
+	to constrain to.
+
+	The passed width and height are in surface coordinate space. If width
+	and height are 0, it means bounds is unknown and equivalent to as if no
+	configure_bounds event was ever sent for this surface.
+
+	The bounds can for example correspond to the size of a monitor excluding
+	any panels or other shell components, so that a surface isn't created in
+	a way that it cannot fit.
+
+	The bounds may change at any point, and in such a case, a new
+	xdg_toplevel.configure_bounds will be sent, followed by
+	xdg_toplevel.configure and xdg_surface.configure.
+      
+  */
+  std::function<void(int32_t, int32_t)> &on_configure_bounds();
+
+};
+
+/** \brief 
+
+  */
+enum class xdg_toplevel_error : uint32_t
+  {
+  /** \brief provided value is         not a valid variant of the resize_edge enum */
+  invalid_resize_edge = 0
 };
 
 /** \brief edge values for resizing
@@ -1620,21 +1700,21 @@ public:
 	is being dragged in a resize operation.
       
   */
-struct xdg_toplevel_resize_edge : public detail::bitfield<4, 10>
+struct xdg_toplevel_resize_edge : public detail::bitfield<4, 11>
 {
-  xdg_toplevel_resize_edge(const detail::bitfield<4, 10> &b)
-    : detail::bitfield<4, 10>(b) {}
+  xdg_toplevel_resize_edge(const detail::bitfield<4, 11> &b)
+    : detail::bitfield<4, 11>(b) {}
   xdg_toplevel_resize_edge(const uint32_t value)
-    : detail::bitfield<4, 10>(value) {}
-  static const detail::bitfield<4, 10> none;
-  static const detail::bitfield<4, 10> top;
-  static const detail::bitfield<4, 10> bottom;
-  static const detail::bitfield<4, 10> left;
-  static const detail::bitfield<4, 10> top_left;
-  static const detail::bitfield<4, 10> bottom_left;
-  static const detail::bitfield<4, 10> right;
-  static const detail::bitfield<4, 10> top_right;
-  static const detail::bitfield<4, 10> bottom_right;
+    : detail::bitfield<4, 11>(value) {}
+  static const detail::bitfield<4, 11> none;
+  static const detail::bitfield<4, 11> top;
+  static const detail::bitfield<4, 11> bottom;
+  static const detail::bitfield<4, 11> left;
+  static const detail::bitfield<4, 11> top_left;
+  static const detail::bitfield<4, 11> bottom_left;
+  static const detail::bitfield<4, 11> right;
+  static const detail::bitfield<4, 11> top_right;
+  static const detail::bitfield<4, 11> bottom_right;
 };
 
 /** \brief types of state on the surface
@@ -1689,12 +1769,6 @@ enum class xdg_toplevel_state : uint32_t
       The parent of an xdg_popup must be mapped (see the xdg_surface
       description) before the xdg_popup itself.
 
-      The x and y arguments passed when creating the popup object specify
-      where the top left of the popup should be placed, relative to the
-      local surface coordinates of the parent surface. See
-      xdg_surface.get_popup. An xdg_popup must intersect with or be at least
-      partially adjacent to its parent surface.
-
       The client must call wl_surface.commit on the corresponding wl_surface
       for the xdg_popup state to take effect.
     
@@ -1706,9 +1780,10 @@ private:
   {
     std::function<void(int32_t, int32_t, int32_t, int32_t)> configure;
     std::function<void()> popup_done;
+    std::function<void(uint32_t)> repositioned;
   };
 
-  static int dispatcher(uint32_t opcode, std::vector<detail::any> args, std::shared_ptr<detail::events_base_t> e);
+  static int dispatcher(uint32_t opcode, const std::vector<detail::any>& args, const std::shared_ptr<detail::events_base_t>& e);
 
   xdg_popup_t(proxy_t const &wrapped_proxy, construct_proxy_wrapper_tag /*unused*/);
 
@@ -1770,12 +1845,52 @@ public:
 	will always have keyboard focus.
       
   */
-  void grab(seat_t seat, uint32_t serial);
+  void grab(seat_t const& seat, uint32_t serial);
 
   /** \brief Minimum protocol version required for the \ref grab function
   */
   static constexpr std::uint32_t grab_since_version = 1;
 
+  /** \brief recalculate the popup's location
+      \param positioner 
+      \param token reposition request token
+
+	Reposition an already-mapped popup. The popup will be placed given the
+	details in the passed xdg_positioner object, and a
+	xdg_popup.repositioned followed by xdg_popup.configure and
+	xdg_surface.configure will be emitted in response. Any parameters set
+	by the previous positioner will be discarded.
+
+	The passed token will be sent in the corresponding
+	xdg_popup.repositioned event. The new popup position will not take
+	effect until the corresponding configure event is acknowledged by the
+	client. See xdg_popup.repositioned for details. The token itself is
+	opaque, and has no other special meaning.
+
+	If multiple reposition requests are sent, the compositor may skip all
+	but the last one.
+
+	If the popup is repositioned in response to a configure event for its
+	parent, the client should send an xdg_positioner.set_parent_configure
+	and possibly an xdg_positioner.set_parent_size request to allow the
+	compositor to properly constrain the popup.
+
+	If the popup is repositioned together with a parent that is being
+	resized, but not in response to a configure event, the client should
+	send an xdg_positioner.set_parent_size request.
+      
+  */
+  void reposition(xdg_positioner_t const& positioner, uint32_t token);
+
+  /** \brief Minimum protocol version required for the \ref reposition function
+  */
+  static constexpr std::uint32_t reposition_since_version = 3;
+
+  /** \brief Check whether the \ref reposition function is available with
+      the currently bound version of the protocol
+  */
+  bool can_reposition() const;
+
   /** \brief configure the popup surface
       \param x x position relative to parent surface window geometry
       \param y y position relative to parent surface window geometry
@@ -1789,6 +1904,11 @@ public:
 	The x and y arguments represent the position the popup was placed at
 	given the xdg_positioner rule, relative to the upper left corner of the
 	window geometry of the parent surface.
+
+	For version 2 or older, the configure event for an xdg_popup is only
+	ever sent once for the initial configuration. Starting with version 3,
+	it may be sent again if the popup is setup with an xdg_positioner with
+	set_reactive requested, or in response to xdg_popup.reposition requests.
       
   */
   std::function<void(int32_t, int32_t, int32_t, int32_t)> &on_configure();
@@ -1802,6 +1922,28 @@ public:
   */
   std::function<void()> &on_popup_done();
 
+  /** \brief signal the completion of a repositioned request
+      \param token reposition request token
+
+	The repositioned event is sent as part of a popup configuration
+	sequence, together with xdg_popup.configure and lastly
+	xdg_surface.configure to notify the completion of a reposition request.
+
+	The repositioned event is to notify about the completion of a
+	xdg_popup.reposition request. The token argument is the token passed
+	in the xdg_popup.reposition request.
+
+	Immediately after this event is emitted, xdg_popup.configure and
+	xdg_surface.configure will be sent with the updated size and position,
+	as well as a new configure serial.
+
+	The client should optionally update the content of the popup, but must
+	acknowledge the new popup configuration for the new position to take
+	effect. See xdg_surface.ack_configure for details.
+      
+  */
+  std::function<void(uint32_t)> &on_repositioned();
+
 };
 
 /** \brief 
